<?php
/*
 * (c) Logiless Inc.
 */

namespace Plugin\Logiless\Controller;

use Doctrine\ORM\NonUniqueResultException;
use Doctrine\ORM\QueryBuilder;
use Eccube\Application;
use Eccube\Entity\Member;
use Plugin\Logiless\Exception\LogilessException;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Class AbstractController
 * @package Plugin\Logiless\Controller
 */
abstract class AbstractController implements LogilessControllerInterface
{
    /**
     * @param Application $app
     * @param Request $request
     *
     * @return Response
     */
    public function index(Application $app, Request $request)
    {
        try {
            $this->auth($app, $request);

            return $this->process($app, $request);

        } catch (LogilessException $le) {
            return $this->createResponse(array('error_code' => $le->getErrorCode()), $le->getHttpStatus());
        }
    }

    /**
     * 認証
     *
     * @param Application $app
     * @param Request $request
     *
     * @throws LogilessException
     */
    protected function auth(Application $app, Request $request)
    {
        // HTTPヘッダーから認証情報を取得
        $idInfo = explode(':', $request->headers->get('X-LogilessToken'));

        if (2 !== count($idInfo)) {
            throw LogilessException::onAccessDenied(LogilessException::ERROR_AUTH_INVALID_TOKEN);
        }

        try {
            // メンバーを検索
            $qb = $app['orm.em']->createQueryBuilder();
            /* @var $qb QueryBuilder */

            $member = $qb
                ->select('m')
                ->from('Eccube\Entity\Member', 'm')
                ->where('m.login_id = :login_id')
                ->setParameter('login_id', $idInfo[0])
                ->getQuery()
                ->getOneOrNullResult();

            if (!$member instanceof Member) {
                throw LogilessException::onAccessDenied(LogilessException::ERROR_AUTH_INVALID_TOKEN);
            }

            $encodedPassword = $app['eccube.password_encoder']->encodePassword($idInfo[1], $member->getSalt());

            if ($encodedPassword !== $member->getPassword()) {
                throw LogilessException::onAccessDenied(LogilessException::ERROR_AUTH_INVALID_TOKEN);
            }

        } catch (NonUniqueResultException $e) {
            throw LogilessException::onAccessDenied(LogilessException::ERROR_AUTH_LOGIN_ID_IS_DUPLICATED);
        }
    }

    /**
     * JsonResponseを生成
     *
     * @param mixed $data
     * @param int $status
     * @param array $headers
     *
     * @return JsonResponse
     */
    protected function createResponse($data, $status = 200, $headers = array())
    {
        return new JsonResponse($data, $status, $headers);
    }

    /**
     * @param Application $app
     * @param Request $request
     *
     * @return Response
     */
    abstract protected function process(Application $app, Request $request);
}
